﻿using System;
using System.ComponentModel;

public partial class AdvancedPagingControl : System.Web.UI.UserControl
{
    // 몇번째 페이지를 보여줄 건지 : 웹 폼에서 전달됨
    private int _PageIndex;
    [Category("페이징처리")]
    public int PageIndex 
    {
        get { return _PageIndex; }
        set { _PageIndex = value; }
    }
    // 총 몇개의 페이지가 만들어지는지 : 총 레코드 수 / 10(한페이지에서 보여줄)
    private int _PageCount;
    [Category("페이징처리")]
    public int PageCount
    {
        get { return _PageCount; }
        set { _PageCount = value; }
    }
    // 페이지 사이즈 : 한 페이지에 몇개의 레코드를 보여줄건지 결정
    private int _PageSize;
    [Category("페이징처리")]
    [Description("한 페이지에 몇개의 레코드를 보여줄건지 결정")]
    public int PageSize {
        get { return _PageSize; }
        set { _PageSize = value; }
    }
    // 레코드 카운트 : 현재 테이블에 몇개의 레코드가 있는지 지정
    private int _RecordCount;
    [Category("페이징처리")]
    [Description("현재 테이블에 몇개의 레코드가 있는지 지정")]
    public int RecordCount {
        get { return _RecordCount; }
        set {
            _RecordCount = value;
            PageCount = ((_RecordCount - 1) / _PageSize) + 1; // 계산식
        }
    }

    // 검색 필드
    [Category("페이징처리")]
    [Description("검색 필드 : Name, Title, Content")]    
    public string SearchField { get; set; }

    // 검색어
    [Category("페이징처리")]
    [Description("검색할 단어")]
    public string SearchQuery { get; set; }

    // Public Constructors
    public AdvancedPagingControl() {
        _PageSize = 10; // 페이지 사이즈는 기본값이 10
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        AdvancedPaging(PageIndex, PageCount); // 페이징 처리
    }
    /// <summary>
    /// 고급(성능향상) 페이징 처리 메서드
    /// </summary>
    /// <param name="PageNo">페이지번호 : PageIndex</param>
    /// <param name="NumPage">페이지수 : PageCount</param>
    private void AdvancedPaging(int PageNo, int NumPage)
    {
        ++PageNo;
        int i = 0;
        string strPage = "<!--이전 10개, 다음 10개 페이징 처리 시작--><font style=\"font-size: 9pt;\"><font color=\"#c0c0c0\">[</font>";
        if (PageNo > 10)
        {
            strPage += "<a href=\"" + Request.ServerVariables["SCRIPT_NAME"] 
                + "?SearchField=" + SearchField + "&SearchQuery=" + SearchQuery + "&Page=" + Convert.ToString(((PageNo - 1) / (int)10) * 10) + "\">◀</a>";
        }
        else
        {
            strPage += "<font color=\"#c0c0c0\">◁</font>";
        }

        strPage += "<font color=\"#c0c0c0\">|</font>";

        for (i = (((PageNo - 1) / (int)10) * 10 + 1); i <= ((((PageNo - 1) / (int)10) + 1) * 10); i++)
        {
            if (i > NumPage)
            {
                break;
            }
            if (i == PageNo)
            {
                strPage += "<b>" + i.ToString() + "</b> <font color=\"#c0c0c0\">|</font>";
            }
            else
            {
                strPage += "<a href=\"" + Request.ServerVariables["SCRIPT_NAME"]
                    + "?SearchField=" + SearchField + "&SearchQuery=" + SearchQuery + "&Page=" + i.ToString() + "\">" + i.ToString() + "</a> <font color=\"#c0c0c0\">|</font>";
            }
        }

        if (i < NumPage)
        {
            strPage += "<a href=\"" + Request.ServerVariables["SCRIPT_NAME"]
                + "?SearchField=" + SearchField + "&SearchQuery=" + SearchQuery + "&Page=" + Convert.ToString(((PageNo - 1) / (int)10) * 10 + 11) + "\">▶</a>";
        }
        else
        {
            strPage += "<font color=\"#c0c0c0\">▷</font>";
        }

        strPage += "<font color=\"#c0c0c0\">]</font></font><!--이전 10개, 다음 10개 페이징 처리 종료-->";

        ctlAdvancedPaingList.Text = strPage;
    }
}
